package fr.username404.snowygui.gui.elements

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.DefaultVertexFormat
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.Renderable.Rendering.buffer
import fr.username404.snowygui.gui.Renderable.Rendering.defaultRectFunc
import fr.username404.snowygui.gui.Renderable.Rendering.tessellator
import fr.username404.snowygui.gui.elements.ClickButton.Companion.Type
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import net.minecraft.client.Minecraft
import net.minecraft.network.chat.TranslatableComponent
import org.lwjgl.opengl.GL20

class ClickBox(
    x: Double, y: Double,
    color: Int = 0x6C9E9D,
    private val name: TranslatableComponent? = null
): ColoredElement(x, y, 80, 10, color, 0.5F) {
    val buttons = mutableMapOf<String, ClickButton>() // Can contain up to 16 buttons
    fun addButtons(vararg collect: Pair<String, (() -> Unit)?>, kind: Type = Type.TOGGLE): ClickBox {
        buttons.putAll(
            collect.map {
                it.first to ClickButton(title = it.first, action = it.second, getColorFrom = this, kind = kind)
            }
        )
        return this
    }
    @JvmField
    val buttonsProgressBar: ColoredElement = object: ColoredElement(
        (x + width), y + height + 3,
        color = color, width = 3, height = clickboxHeightOffset - 6,
        opacity = 0.75F
    ) {
        override fun render(poseStack: PoseStack?) {
            RenderSystem.disableTexture()
            RenderSystem.enableBlend()
            RenderSystem.defaultBlendFunc()
            defaultRectFunc()
            RenderSystem.disableBlend()
            RenderSystem.enableTexture()
        }
        init { height = 8 }
    }
    var barStage: Int = 1; private set
    fun scroll(d: Double, e: Double, supplied: Double) {
        if (withinBounds(d, e, offsetHeight = clickboxHeightOffset.toDouble())) {
            with(buttonsProgressBar) {
                if ((height != 8 || (supplied < 0)) && ((height != originalHeight) || (supplied > 0))) {
                    height -= supplied.toInt()
                    if (buttons.isNotEmpty()) (height / 8).let {
                        if (it > 0) barStage = it
                    }
                }
            }
        }
    }

    companion object {
        const val clickboxHeightOffset: Int = 80
        private const val inclination: Double = 2.5
    }
    override fun render(poseStack: PoseStack?) {
        runBlocking {
            RenderSystem.disableTexture()
            RenderSystem.enableBlend()
            RenderSystem.defaultBlendFunc()
            with(buffer) {
                begin(GL20.GL_POLYGON, DefaultVertexFormat.POSITION_COLOR)
                // Render the header:
                vertex(x, y + height, 0.0).colorEnd()
                vertex(x + width + inclination, y + height, 0.0).colorEnd()
                vertex(x + width, y, 0.0).colorEnd()
                vertex(x + inclination, y, 0.0).colorEnd()

                // Render the box:
                val currentHeight = y + (height + clickboxHeightOffset)
                vertex(x, currentHeight, 0.0).colorEnd()
                vertex(x + width + inclination, currentHeight, 0.0).colorEnd()
                vertex(x + width + inclination, y + height, 0.0).colorEnd()
                tessellator.end()

                begin(GL20.GL_LINES, DefaultVertexFormat.POSITION_COLOR)
                vertex(x + inclination, y + height, 0.0).colorEnd(0xF2F2FC)
                vertex(x + width, y + height, 0.0).colorEnd(0xF2F2FC)
                tessellator.end()
            }
            RenderSystem.enableTexture()
            RenderSystem.disableBlend()

            val renderButtons = if (buttons.isNotEmpty()) launch {
                buttonsProgressBar.apply {
                    x = this@ClickBox.x + this@ClickBox.width - 3
                    y = this@ClickBox.y + this@ClickBox.height + 3
                }.display(poseStack)
                buttons.values.forEachIndexed { num, button ->
                    val fullHeight = (y + height.toDouble())..(y + height + clickboxHeightOffset)
                    button.also {
                        it.x = x + 3
                        it.y = y + 3 + height + (((num + 1) - barStage) * 9)
                        it.hidden = if ((num + 1) <= 8) ((it.y) !in fullHeight) else ((it.y + it.height) !in fullHeight)
                    }.display(poseStack)
                }
            } else null
            if ((name != null) && (poseStack != null)) {
                Minecraft.getInstance().font.draw(poseStack, name.string, x.toFloat() + 5, y.toFloat() + 1.5F, TransparentColor)
                renderButtons?.join()
            }
        }
    }
}